/* 
 * $Id: MyXDTPModule.h,v 1.2 2005/10/17 10:18:36 ozawa Exp $
 *
 * XDTP external sample module program.
 *
 * Copyright 2005- ONGS Inc.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.2 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#ifndef ___MYXDTPMODULE_H
#define ___MYXDTPMODULE_H

#include "modules/XDTPModule.h"

class MyXDTPModule : public XDTP::XDTPModule
{
public:
	MyXDTPModule();
	virtual ~MyXDTPModule();

    /**
     * Set an output type.
     * 
     * @param aType
	 *            output type
     */
    virtual void setOutputType(const Glib::ustring& aType);
    
    /**
     * Set options for modules.
     * 
     * @param anOptions
	 *            options
     */
    virtual void setOption(const XDTP::StringList& anOptions);

    /**
     * Set the output file.
     * 
     * @param anOutputFile
	 *            output file
     */
    virtual void setOutputFile(const Glib::ustring& anOutputFile);

    /**
     * Set the input xml file.
     * 
     * @param aXMLFile
	 *            input xml file
     */
    virtual void setXMLFile(const Glib::ustring& aXMLFile);
    
    /**
     * Set an input file encoding.
	 * This is not the input xml file encoding.
     * 
     * @param anInputEncoding
	 *            input file encoding
     */
    virtual void setInputEncoding(const Glib::ustring& anInputEncoding);

    /**
     * Set an output file encoding.
     * 
     * @param anOutputEncoding
	 *            output file encoding
     */
    virtual void setOutputEncoding(const Glib::ustring& anOutputEncoding);

    /**
     * Callback before parse the input xml file.
     * 
     * @param aFile
	 *            input xml file
     */
    virtual void treatPreFile(const Glib::ustring& aFile);

    /**
     * Callback after parse the input xml file.
     * 
     * @param aDocument
	 *            xml document
     */
    virtual void treatPreDocument(xmlDocPtr aDocument);

    /**
     * Callback after XSLT.
     * 
     * @param aDocument
	 *            xml document
     */
    virtual void treatPostDocument(xmlDocPtr aDocument);

    /**
     * Callback after save.
     * 
     * @param aFile
	 *            output file
     */
    virtual void treatPostFile(const Glib::ustring& aFile);

protected:
	Glib::ustring m_OutputFile;
	Glib::ustring m_OutputEncoding;
};

#endif
