/* 
 * $Id: ImportTextCSV.cpp,v 1.17 2005/11/02 08:13:37 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.17 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "xdtp.h"
#include "modules/imports/ImportTextCSV.h"
#include "CSVAnalyzer.h"
#include "Utilities.h"
#include "URLTool.h"
#include "XMLTool.h"

using namespace XDTP;

ImportTextCSV::ImportTextCSV()
{
}

ImportTextCSV::~ImportTextCSV()
{
}

/**
 * XSLѴXMLʸϤ¸ľ˸ƤӽФޤ
 * 
 * @param aDocument XMLʸ
 */
void ImportTextCSV::treatPostDocument(xmlDocPtr aDocument)
{
    GLSDImportModuleAdapter::treatPostDocument(aDocument);

    int count;
    XMLTool tool;
    xmlNodePtr root = xmlDocGetRootElement(aDocument);
    xmlNodeSetPtr nodes;
    xmlXPathObjectPtr obj = tool.getNodeList(root, "//import[@type=\"text/csv\"]");

    if (obj) {
        nodes = obj->nodesetval;
        
        for (count = 0; count < nodes->nodeNr; count++) {
            if (!transform(aDocument, nodes->nodeTab[count])) {
                fprintf(stderr, "%s: ImportTextCSV: transform failed.\n", APP_NAME);
            }
        }
        
        xmlXPathFreeObject(obj);
    }
}

bool ImportTextCSV::transform(xmlDocPtr aDocument, xmlNodePtr aNode)
{
    Glib::ustring ref;
    Glib::ustring caption;
    Glib::ustring encoding;

    if (!getImportAttributes(aNode, ref, caption, encoding)) {
        return false;
    }
    
    if ("html" == m_OutputType) {
        return csvTo_nHTML(true, aDocument, aNode, ref, caption, encoding);
    }
    else if ("xhtml" == m_OutputType) {
        return csvTo_nHTML(false, aDocument, aNode, ref, caption, encoding);
    }
    else if ("text" == m_OutputType) {
        return preserveImport(aDocument, aNode, ref, caption, encoding);
    }
    // otherwise, do not operation.
    
    return true;
}

bool ImportTextCSV::csvTo_nHTML
    (bool isHTML, xmlDocPtr aDocument, xmlNodePtr aNode,
     const Glib::ustring& aRef,
     const Glib::ustring& aCaption,
     const Glib::ustring& anEncoding)
{
    // 륷Ȥ
    xmlNodePtr comment = getStyleSheet(aDocument, "text/css");

    if (comment) {
        // 륷Ȥ˥᡼Υλ꤬뤫ǧ
        Glib::ustring content = (comment->content ? (const char*)comment->content : "");
        Glib::ustring::size_type pos = content.find("table.csv");
        
        if (Glib::ustring::npos == pos) {
            // 륷Ȥ˥᡼Υɲ
            content += LINE_SEP;
            content += "   table.csv {" LINE_SEP;
            content += "     border-collapse: collapse;" LINE_SEP;
            content += "     margin: 1.5mm auto 3mm auto;" LINE_SEP;
            content += "     empty-cells: show;" LINE_SEP;
            content += "     caption-side: top;" LINE_SEP;
            content += "     width: 90%;" LINE_SEP;
            content += "   }" LINE_SEP;
            content += "   table.csv tr th," LINE_SEP;
            content += "   table.csv tr td {" LINE_SEP;
            content += "     border: 2px solid gray;" LINE_SEP;
            content += "   }" LINE_SEP;
            content += "   caption.csv {" LINE_SEP;
            content += "     margin-top: 3mm;" LINE_SEP;
            content += "   }" LINE_SEP;
            xmlNodeSetContent(comment, (const xmlChar*)content.c_str());
        }
    }
    
    // ơ֥륨Ⱥ
    xmlNodePtr table = xmlNewNode(NULL, (const xmlChar*)"table");
    xmlSetProp(table, (const xmlChar*)"border", (const xmlChar*)"1");
    xmlSetProp(table, (const xmlChar*)"class", (const xmlChar*)"csv");

    // ɽ
    xmlNodePtr caption = xmlNewNode(NULL, (const xmlChar*)"caption");
    xmlSetProp(caption, (const xmlChar*)"class", (const xmlChar*)"csv");
    if (!aCaption.empty()) {
        xmlAddChild(caption, xmlNewText((const xmlChar*)aCaption.c_str()));
    }
    xmlAddChild(table, caption);

    // csvβϤȥơ֥빽¤ι
    bool bFailed = false;
    std::string tmpfile;
    try {
        Glib::RefPtr<Glib::IOChannel> ioc = getIOChannel(aRef, tmpfile);
        setEncoding(ioc, anEncoding);
        
        Glib::ustring line;
        StringList list;
        StringList::iterator iter, end;
        CSVAnalyzer analyzer;
        xmlNodePtr tr, th, td;
        
        // ܤ̾Ȥư
        if (ioc->read_line(line) == Glib::IO_STATUS_NORMAL) {
            line = Utilities::strTrimCRLF(line);
            if (!analyzer.analyzeLine(line, list)) {
                ioc->close();
                THROW("CSV data is broken. : " + line);
            }
            
            tr = xmlNewNode(NULL, (const xmlChar*)"tr");
            xmlAddChild(table, tr);

            end = list.end();
            for (iter = list.begin(); iter != end; iter++) {
                th = xmlNewNode(NULL, (const xmlChar*)"th");
                xmlAddChild(th, xmlNewText((const xmlChar*)(*iter).c_str()));
                xmlAddChild(tr, th);
            }
        }

        // ܰʹߤν
        while (ioc->read_line(line) == Glib::IO_STATUS_NORMAL) {
            line = Utilities::strTrimCRLF(line);

            if (!analyzer.analyzeLine(line, list)) {
                ioc->close();
                THROW("CSV data is broken. : " + line);
            }

            tr = xmlNewNode(NULL, (const xmlChar*)"tr");
            xmlAddChild(table, tr);

            end = list.end();
            for (iter = list.begin(); iter != end; iter++) {
                td = xmlNewNode(NULL, (const xmlChar*)"td");
                xmlAddChild(tr, td);

                // ʪͤǤ籦ǽϤ褦˽
                if (isNumber((*iter))) {
                    xmlAddChild(td, xmlNewText((const xmlChar*)(*iter).c_str()));
                    xmlSetProp(td, (const xmlChar*)"align", (const xmlChar*)"right");
                    continue;
                }
                // ʪϥѡ󥯤оݤǤϥѡ
                // ǽϤ褦˽
                else if (isHyperLink((*iter))) {
                    // ','', 'ǶڤƤ硢ʣΥ
                    // ʣʸ󤫤äƤΤȤƽ
                    // ɬפ롣ä, ʸ ','
                    // ˤäʬ䤷ʬ䤷Τ줾Ф
                    // Ԥ
                    StringList tmplist;
                    StringList::iterator tmpbegin, tmpiter, tmpend;
                    xmlNodePtr a;
                    if (analyzer.analyzeLine((*iter), tmplist)) {
                        tmpend = tmplist.end();
                        for (tmpbegin = tmpiter = tmplist.begin(); tmpiter != tmpend; tmpiter++) {
                            if (tmpiter != tmpbegin) {
                                xmlAddChild(td, xmlNewText((const xmlChar*)", "));
                            }
                            if (isHyperLink((*tmpiter))) {
                                a = xmlNewNode(NULL, (const xmlChar*)"a");
                                xmlAddChild(a, xmlNewText((const xmlChar*)(*tmpiter).c_str()));
                                xmlSetProp(a, (const xmlChar*)"href", (const xmlChar*)(*tmpiter).c_str());
                                xmlAddChild(td, a);
                            }
                            else {
                                xmlAddChild(td, xmlNewText((const xmlChar*)(*tmpiter).c_str()));
                            }
                        }
                        continue;
                    }
                }

                xmlAddChild(td, xmlNewText((const xmlChar*)(*iter).c_str()));
            }
        }

        ioc->close();
    }
    catch (Exception exception) {
        exception.print(stderr);
        bFailed = true;
    }
    catch (Glib::Error error) {
        std::string msg = Glib::locale_from_utf8(error.what());
        fprintf(stderr, "%s: %s (%s)\n", APP_NAME, msg.c_str(), aRef.c_str());
        bFailed = true;
    }
    catch (std::exception exception) {
        fprintf(stderr, "%s: %s (%s)\n", APP_NAME, exception.what(), aRef.c_str());
        bFailed = true;
    }
    catch (...) {
        fprintf(stderr, "%s: ImportTextCSV: unknown error occured. (%s)\n",
            APP_NAME, aRef.c_str());
        bFailed = true;
    }

    if (0 < tmpfile.length()) {
        remove(tmpfile.c_str());
    }
    if (bFailed) return false;
    
    // import  table 
    xmlFreeNode(xmlReplaceNode(aNode, table));

    return true;
}

bool ImportTextCSV::isNumber(const Glib::ustring& aString)
{
    Glib::ustring str = aString.lowercase();
    
    // Ƭ +/- å
    if (0 >= str.length()) return false;
    if ("+" == str.substr(0, 1) || "-" == str.substr(0, 1)) {
        str = str.substr(1);
    }
    
    // ͥå
    if (0 >= str.length()) return false;

    bool bDot = false;
    bool bOK = false;
    Glib::ustring::iterator iter = str.begin();
    Glib::ustring::iterator end = str.end();
    
    for (; iter != end; iter++) {
        if (('0' <= *iter && '9' >= *iter) ||
            (0xb0a3 <= *iter && 0xb9a3 >= *iter))
        {
            bOK = true;
        }
        else if ('.' == *iter || 0xa5a1 == *iter) {
            if (bDot) return false;
            bDot = true;
        }
        else if (',' == *iter || 0xa4a1 == *iter) {
            // ޤ̵롣ʹߤΥޤϵݡ
            if (bDot) return false;
        }
        else if (0xc1a1 == *iter) {
            // ü''ϵ
            iter++;
            if (iter != end) return false;
            break;
        }
        else {
            return false;
        }
    }
    
    return bOK;
}

bool ImportTextCSV::isHyperLink(const Glib::ustring& aString)
{
    int len = aString.length();
    Glib::ustring str = aString.lowercase();
    
    if (6 < len && "ftp://" == str.substr(0, 6)) return true;
    else if (7 < len && "http://" == str.substr(0, 7)) return true;
    else if (8 < len && "https://" == str.substr(0, 8)) return true;
    
    return false;
}
