/* 
 * $Id: XDTPTransform.cpp,v 1.16 2005/11/13 03:45:28 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.16 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "xdtp.h"
#include "XDTPTransform.h"

#include "Exception.h"
#include "Utilities.h"
#include "xmlloader.h"
#include "xmlsaver.h"
#include "XMLTool.h"

#include "modules/GLSDModule.h"
#include "modules/GLSDModuleChecker.h"
#include "modules/GLSDDocumentModule.h"
#include "modules/GLSDDocumentModuleChecker.h"
#include "modules/GLSDBanBunModule.h"
#include "modules/GLSDBanBunModuleChecker.h"

#define RESOURCE_TAG    "res:"

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace XDTP {
static void objectwrapper_ustring_destractor(void *value)
{
    Glib::ustring *string = (Glib::ustring*)value;
    if (string) delete string;
}

static void objectwrapper_mc_destractor(void *value)
{
    XDTP_MC *mc = (XDTP_MC*)value;
    if (mc) delete mc;
}

static void objectwrapper_module_destractor(void *value)
{
    XDTPModule *module = (XDTPModule*)value;
    if (module) delete module;
}

static void objectwrapper_checker_destractor(void *value)
{
    XDTPChecker *checker = (XDTPChecker*)value;
    if (checker) delete checker;
}
};

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
using namespace XDTP;

XDTPTransform::XDTPTransform()
{
    m_XSL4OutputPos = -1;
    m_bSkipXMLCheck = false;
    m_nVerbose = 0;
    m_OutputType = DEFAULT_OUTPUT_TYPE;
    m_OutputEncoding = DEFAULT_OUTPUT_ENCODING;
    
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_USTRING, objectwrapper_ustring_destractor);
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_XSL, objectwrapper_null_destractor);
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_MC, objectwrapper_mc_destractor);
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_XDTP_EXT_MODULE, objectwrapper_null_destractor);
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_XDTP_EXT_CHECKER, objectwrapper_null_destractor);
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_XDTP_LCL_MODULE, objectwrapper_module_destractor);
    XDTPObjectWrapper::setDestractor(OBJECT_TYPE_XDTP_LCL_CHECKER, objectwrapper_checker_destractor);
}

XDTPTransform::~XDTPTransform()
{
    uninitialize();
}

/**
 * Ԥޤ
 *  set/get ϤΥ᥽åɤѤ뤳Ȥ򤱤Ƥ
 * Ѵ set/get ϥ᥽åɤѤ뤳ȤϤǤޤ
 * 
 * @return  true ᤷޤ
 */
bool XDTPTransform::initialize()
{
    uninitialize();
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // ⥸塼Υ
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: load modules.\n", APP_NAME);
        fflush(stdout);
    }

    if (!loadModules(m_libraries, m_modules, m_checkers)) {
        return false;
    }

    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // ⥸塼ν
    int count = 0;
    const int size = m_modules.size();
    XDTPModule *module = NULL;
    
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: initiailze modules.\n", APP_NAME);
        fflush(stdout);
    }

    for (; count < size; count++) {
        module = (XDTPModule*)m_modules.get(count);
        module->setOutputType(m_OutputType);
        module->setOption(m_OptionList);
        module->setInputEncoding(m_InputEncoding);
        module->setOutputEncoding(m_OutputEncoding);
    }
    
    return true;
}

/**
 * ֥ȤԤޤ
 * Ѵ塢֥ȤѤ뤳Ȥʤ䡢֤ѹ
 * ϡ˸ƤӽФɬפޤ
 */
void XDTPTransform::uninitialize()
{
    unloadModules(m_libraries, m_modules, m_checkers);
}

/**
 * Ѵ¹Ԥޤ
 * 
 * @param anInput ѴXMLե
 * @param anOutput եʴ¸ΥեϾ񤭤ޤ
 * @return Ѵ true ᤷޤ
 */
bool XDTPTransform::transform(const Glib::ustring& anInput, const Glib::ustring& anOutput)
{
    bool result = true;
    
    XMLTool xmltool;
    
    // 顼å򥯥ꥢ
    m_ErrorMsg.clear();
    
    if (0 < m_nVerbose) {
        fprintf(stdout, "%s: transform: %s\n", APP_NAME, anInput.c_str());
        fflush(stdout);
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // ⥸塼ϥեΤ
    int count = 0;
    const int modules_size = m_modules.size();
    XDTPModule *module = NULL;
    
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: set input/output file to modules.\n", APP_NAME);
        fflush(stdout);
    }

    for (; count < modules_size; count++) {
        module = (XDTPModule*)m_modules.get(count);
        module->setOutputFile(anOutput);
        module->setXMLFile(anInput);
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // ϥեʣ
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: backup the input file.\n", APP_NAME);
        fflush(stdout);
    }

    std::string tmpfile = "_xdtp_temp.XXXXXX";
    int fd = Glib::mkstemp(tmpfile);
    
    if (0 > fd) {
        setErrorMsg("Unable to create temp file.");
        return false;
    }
    
    if (!Utilities::copy(anInput, fd)) {
        setErrorMsg("Unable to backup the input file. (" + anInput + ")");
        close(fd);
        remove(tmpfile.c_str());
        return false;
    }
    close(fd);
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // parse
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: treatPreFile.\n", APP_NAME);
        fflush(stdout);
    }

    for (count = 0; count < modules_size; count++) {
        module = (XDTPModule*)m_modules.get(count);
        module->treatPreFile(tmpfile);
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // parse
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: parse the XML file.\n", APP_NAME);
        fflush(stdout);
    }

    xmlDocPtr document = XMLLoader::load(tmpfile);
    
    if (!document) {
        setErrorMsg("Unable to parse the input xml file.");
        remove(tmpfile.c_str());
        return false;
    }
    remove(tmpfile.c_str());
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // check
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: check the XML file.\n", APP_NAME);
        fflush(stdout);
    }

    if (!m_bSkipXMLCheck) {
        const int checkers_size = m_checkers.size();
        XDTPChecker *checker = NULL;
        
        for (count = 0; count < checkers_size; count++) {
            checker = (XDTPChecker*)m_checkers.get(count);
            if (!checker->checkFormat(document)) {
                setErrorMsg("checkFormat failed.");
                xmlFreeDoc(document);
                return false;
            }
        }
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // parse
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: treatPreDocument.\n", APP_NAME);
        fflush(stdout);
    }

    for (count = 0; count < modules_size; count++) {
        module = (XDTPModule*)m_modules.get(count);
        module->treatPreDocument(document);
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // xslt
    {
        if (1 < m_nVerbose) {
            fprintf(stdout, "%s: transform: do xslt.\n", APP_NAME);
            fflush(stdout);
        }

        int count = 0;
        const int size = m_XSLObjects.size();
        xmlDocPtr doctmp = NULL;
        xsltStylesheetPtr xsl = NULL;
        
        for (; document && count < size; count++) {
            doctmp = NULL;
            xsl = parseXSL(count);
            
            if (xsl) {
                doctmp = xmltool.transform(document, xsl);
                xmlFreeDoc(document);
                xsltFreeStylesheet(xsl);
                document = doctmp;
            }
            
            if (!doctmp) {
                setErrorMsg("XSLT failed.");
            }
        }
    }
    
    if (!document) {
        return false;
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // xsltν
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: treatPostDocument.\n", APP_NAME);
        fflush(stdout);
    }

    for (count = 0; count < modules_size; count++) {
        module = (XDTPModule*)m_modules.get(count);
        module->treatPostDocument(document);
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // ¸
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: save file.\n", APP_NAME);
        fflush(stdout);
    }

    tmpfile = "_xdtp_temp.XXXXXX";
    fd = Glib::mkstemp(tmpfile);
    
    if (0 > fd) {
        setErrorMsg("Unable to create temp file.");
        xmlFreeDoc(document);
        return false;
    }
    
    close(fd);
    
    xsltStylesheetPtr xsl = NULL;
    if (0 < m_XSLObjects.size()) {
        // ϷݻXSLե
        if (0 < m_XSL4OutputPos) xsl = parseXSL(m_XSL4OutputPos);
        if (!xsl) xsl = parseXSL(m_XSLObjects.size() -1);

        // XSLTѴϡ¸˥륷Ⱦɬס
        if (NULL == xsl) {
            setErrorMsg("Unable to load xsl file for output.");
            xmlFreeDoc(document);
            return false;
        }
    }
    if (!XMLSaver::save(tmpfile, document, xsl, m_OutputEncoding)) {
        setErrorMsg("Unable to save xml file. (" + tmpfile + ")");
        if (xsl) xsltFreeStylesheet(xsl);
        xmlFreeDoc(document);
        return false;
    }

    if (xsl) xsltFreeStylesheet(xsl);
    xmlFreeDoc(document);
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // ¸ν
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: treatPostFile.\n", APP_NAME);
        fflush(stdout);
    }

    for (count = 0; count < modules_size; count++) {
        module = (XDTPModule*)m_modules.get(count);
        module->treatPostFile(tmpfile);
    }
    
    //_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
    // 
    if (1 < m_nVerbose) {
        fprintf(stdout, "%s: transform: output file.\n", APP_NAME);
        fflush(stdout);
    }

    if (0 >= anOutput.length() || anOutput == "-") {
        if (!Utilities::printFile(tmpfile)) {
            setErrorMsg("Unable to print out.");
            result = false;
        }
        remove(tmpfile.c_str());
    }
    else {
        if (!Utilities::move(tmpfile, anOutput)) {
            setErrorMsg("Unable to move file. (" + tmpfile + " -> " + anOutput + ")");
            remove(tmpfile.c_str());
            result = false;
        }
    }
    
    return result;
}

/**
 * Ѵȯ顼åϤޤ
 * 顼ȯʤäϡɽޤ
 * 
 * @param out 
 */
void XDTPTransform::showErrorMsg(FILE* out)
{
    if (0 < m_ErrorMsg.length()) {
        fwrite(m_ErrorMsg.c_str(), 1, m_ErrorMsg.bytes(), out);
        fputs(LINE_SEP, out);
    }
}

/**
 * 顼åꤷޤ
 * 
 * @param aMsg 顼å
 */
void XDTPTransform::setErrorMsg(const Glib::ustring& aMsg)
{
    m_ErrorMsg = aMsg;
}

/**
 * XMLʸϤå򥹥åפޤ
 * 
 * @param bCheck åפ true ꤷޤ
 */
void XDTPTransform::skipXMLCheck(bool bCheck)
{
    m_bSkipXMLCheck = bCheck;
}

/**
 * XMLʸϤå򥹥åפ뤫Ĵ٤ޤ
 * 
 * @return åפ true ᤷޤ
 */
bool XDTPTransform::isXMLCheck()
{
    return m_bSkipXMLCheck;
}

/**
 * ѴXSLեɲäޤ
 * 
 * @param useOutputType ϷݻѤ true ꤷޤ
 */
void XDTPTransform::addXSLFile(const Glib::ustring& aPath, bool useOutputType)
{
    Glib::ustring *path = new Glib::ustring;
    
    if (!path) THROW("Out of memory!");

    *path = aPath;
    m_XSLObjects.add(OBJECT_TYPE_USTRING, path);
    if (useOutputType) {
        m_XSL4OutputPos = m_XSLObjects.size() -1;
    }
}

/**
 * ѴXSL֥Ȥɲäޤ
 * 
 * @param aXSL XSL֥
 * @param useOutputType ֥ȤϷݻѤ true ꤷޤ
 */
void XDTPTransform::addXSLObject(xsltStylesheetPtr aXSL, bool useOutputType)
{
    m_XSLObjects.add(OBJECT_TYPE_XSL, aXSL);
    if (useOutputType) {
        m_XSL4OutputPos = m_XSLObjects.size() -1;
    }
}

/**
 * ѴXSLեΰ򥯥ꥢޤ
 */
void XDTPTransform::clearXSLFile()
{
    m_XSLObjects.clear();
    m_XSL4OutputPos = -1;
}

/**
 * ѴXSLեΰޤ
 * 
 * @deprecated
 * @return ѴXSLեΰ
 */
const StringList& XDTPTransform::getXSLFileList()
{
    int type;
    void *value;

    for (int count = 0; count < m_XSLObjects.size(); count++) {
        value = m_XSLObjects.get(count, &type);
        if (OBJECT_TYPE_USTRING == type) {
            __m_XSLFileList.push_back(*((Glib::ustring*)value));
        }
    }
    
    return __m_XSLFileList;
}

/**
 * ѴXSLե뤪ӥ֥Ȥΰޤ
 * 
 * @return ѴXSLե뤪ӥ֥Ȥΰ
 */
const XDTPObjectWrapper& XDTPTransform::getXSLList()
{
    return m_XSLObjects;
}

/**
 * ѴXDTP⥸塼ɲäޤ
 * 
 * @param aPath XDTP⥸塼ؤΥѥ
 */
void XDTPTransform::addXDTPModule(const Glib::ustring& aPath)
{
    Glib::ustring *path = new Glib::ustring;
    
    if (!path) THROW("Out of memory!");

    *path = aPath;
    m_XDTPModules.add(OBJECT_TYPE_USTRING, path);
}

/**
 * ѴXDTP⥸塼ɲäޤ
 * 
 * @param aModule XDTP⥸塼
 * @param aChecker XDTPåѤʤ NULL Ǥޤ
 */
void XDTPTransform::addXDTPModule(XDTPModule *aModule, XDTPChecker *aChecker)
{
    if (!aModule) THROW("XDTPTransform::addXDTPModule: bad arguments.");

    XDTP_MC *mc = new XDTP_MC;

    if (!mc) THROW("Out of memory!");
    
    mc->module = aModule;
    mc->checker = aChecker;
    
    m_XDTPModules.add(OBJECT_TYPE_MC, mc);
}

/**
 * ѴXDTP⥸塼򥯥ꥢޤ
 */
void XDTPTransform::clearXDTPModule()
{
    m_XDTPModules.clear();
}

/**
 * ѴXDTP⥸塼ΰޤ
 * 
 * @return ѴXDTP⥸塼ΰ
 */
const StringList& XDTPTransform::getXDTPModuleList()
{
    int type;
    void *value;

    for (int count = 0; count < m_XDTPModules.size(); count++) {
        value = m_XDTPModules.get(count, &type);
        if (OBJECT_TYPE_USTRING == type) {
            __m_XDTPModuleList.push_back(*((Glib::ustring*)value));
        }
    }
    
    return __m_XDTPModuleList;
}

/**
 * ѴXDTP⥸塼ΰޤ
 * 
 * @return ѴXDTP⥸塼ΰ
 */
const XDTPObjectWrapper& XDTPTransform::getXDTPMCList()
{
    return m_XDTPModules;
}

/**
 * ѴXDTP⥸塼ؤΥץɲäޤ
 * 
 * @param anOption ɲåץ
 */
void XDTPTransform::addOption(const Glib::ustring& anOption)
{
    m_OptionList.push_back(anOption);
}

/**
 * ѴXDTP⥸塼ؤΥץ򥯥ꥢޤ
 */
void XDTPTransform::clearOption()
{
    m_OptionList.clear();
}

/**
 * ѴXDTP⥸塼ؤΥץΰޤ
 * 
 * @return ѴXDTP⥸塼ؤΥץΰ
 */
const StringList& XDTPTransform::getOptionList()
{
    return m_OptionList;
}

/**
 * Ϸꤷޤ
 * 
 * @param aType Ϸ
 */
void XDTPTransform::setOutputType(const Glib::ustring& aType)
{
    m_OutputType = aType.lowercase();
}

/**
 * Ϸޤ
 * 
 * @return Ϸ
 */
const Glib::ustring& XDTPTransform::getOutputType()
{
    return m_OutputType;
}

/**
 * åɽ٥ꤷޤ
 * 
 * @param aMode åɽ٥
 */
void XDTPTransform::setVerboseMode(int aMode)
{
    m_nVerbose = aMode;
}

/**
 * åɽ٥ޤ
 */
int XDTPTransform::getVerboseMode()
{
    return m_nVerbose;
}

/**
 * ϥեΥ󥳡ɤꤷޤ
 * 
 * @param anEncoding ϥեΥ󥳡
 */
void XDTPTransform::setInputEncoding(const Glib::ustring& anEncoding)
{
    if ("getlocale" == anEncoding.lowercase()) {
        std::string encoding;
        Glib::get_charset(encoding);
        m_InputEncoding = encoding;
    }
    else {
        m_InputEncoding = anEncoding;
    }
}

/**
 * ϥեΥ󥳡ɤޤ
 * 
 * @return ϥեΥ󥳡
 */
const Glib::ustring& XDTPTransform::getInputEncoding()
{
    return m_InputEncoding;
}

/**
 * ϥեΥ󥳡ɤꤷޤ
 * 
 * @param anEncoding ϥեΥ󥳡
 */
void XDTPTransform::setOutputEncoding(const Glib::ustring& anEncoding)
{
    if ("getlocale" == anEncoding.lowercase()) {
        std::string encoding;
        Glib::get_charset(encoding);
        m_OutputEncoding = encoding;
    }
    else {
        m_OutputEncoding = anEncoding;
    }
}

/**
 * ϥեΥ󥳡ɤޤ
 * 
 * @return ϥեΥ󥳡
 */
const Glib::ustring& XDTPTransform::getOutputEncoding()
{
    return m_OutputEncoding;
}

bool XDTPTransform::loadModules(LibraryList& libraries,
                                XDTPObjectWrapper& modules, XDTPObjectWrapper& checkers)
{
    bool result = true;
    int count = 0;
    int type = 0;
    const int size = m_XDTPModules.size();
    void *value;
    XDTP_MC *mc;
    XDTPModuleLibrary *lib = NULL;
    XDTPModule *module = NULL;
    XDTPChecker *checker = NULL;
    Glib::ustring path;
    
    for (; count < size; count++) {
        value = m_XDTPModules.get(count, &type);
        
        if (type == OBJECT_TYPE_MC) {
            mc = (XDTP_MC*)value;
            modules.add(OBJECT_TYPE_XDTP_EXT_MODULE, mc->module);
            if (mc->checker) checkers.add(OBJECT_TYPE_XDTP_EXT_CHECKER, mc->checker);
            
            continue;
        }
        
        if (type != OBJECT_TYPE_USTRING) continue;
        
        path = *((Glib::ustring*)value);
        
        // ⥸塼Υ
        if (4 < path.length() && path.substr(0, 4).lowercase() == RESOURCE_TAG) {
            path = path.substr(4);
            
            result &= loadLocalModule(path, modules, checkers);
            
            continue;
        }
        
        // ⥸塼Υ
        lib = XDTPModuleLoader::loadModuleLibrary(path);
        
        if (!lib) {
            setErrorMsg("Unable to load module. (" + path + ")");
            result = false;
        }
        else {
            module = XDTPModuleLoader::buildModule(lib);
            checker = XDTPModuleLoader::buildChecker(lib);
            
            if (!module) {
                XDTPModuleLoader::unloadModuleLibrary(lib);
                setErrorMsg("Unable to build module. (" + path + ")");
                result = false;
            }
            else {
                libraries.push_back(lib);
                modules.add(OBJECT_TYPE_XDTP_EXT_MODULE, module);
                if (checker) checkers.add(OBJECT_TYPE_XDTP_EXT_CHECKER, checker);
            }
        }
    }
    
    if (!result) {
        unloadModules(libraries, modules, checkers);
    }
    
    return result;
}

void XDTPTransform::unloadModules(LibraryList& libraries,
                                  XDTPObjectWrapper& modules, XDTPObjectWrapper& checkers)
{
    LibraryList::iterator iter = libraries.begin();
    LibraryList::iterator end = libraries.end();
    for (; iter != end; iter++) {
        XDTPModuleLoader::unloadModuleLibrary(*iter);
    }
    libraries.clear();
    
    modules.clear();
    checkers.clear();
}

bool XDTPTransform::loadLocalModule(const Glib::ustring& module,
                                    XDTPObjectWrapper& modules, XDTPObjectWrapper& checkers)
{
    if (module.lowercase() == "glsdmodule") {
        modules.add(OBJECT_TYPE_XDTP_LCL_MODULE, new GLSDModule());
        checkers.add(OBJECT_TYPE_XDTP_LCL_CHECKER, new GLSDModuleChecker());
    }
    else if (module.lowercase() == "glsddocumentmodule") {
        modules.add(OBJECT_TYPE_XDTP_LCL_MODULE, new GLSDDocumentModule());
        checkers.add(OBJECT_TYPE_XDTP_LCL_CHECKER, new GLSDDocumentModuleChecker());
    }
    else if (module.lowercase() == "glsdbanbunmodule") {
        modules.add(OBJECT_TYPE_XDTP_LCL_MODULE, new GLSDBanBunModule());
        checkers.add(OBJECT_TYPE_XDTP_LCL_CHECKER, new GLSDBanBunModuleChecker());
    }
    else {
        setErrorMsg("Unknown module name. (" + module + ")");
        return false;
    }

    return true;
}

xsltStylesheetPtr XDTPTransform::parseXSL(int pos)
{
    int type;
    void *value = m_XSLObjects.get(pos, &type);
    
    switch (type) {
    case OBJECT_TYPE_USTRING:
        return xsltParseStylesheetFile((const xmlChar *)((Glib::ustring*)value)->c_str());
    case OBJECT_TYPE_XSL:
        return (xsltStylesheetPtr)value;
    default:
        THROW("XDTPTransform::parseXSL: Internal error! (Unknown object type)");
    }
    
    return NULL;
}
